@@ -67,6 +67,22 @@ const config = {
           // Please change this to your repo.
           // Remove this to remove the "edit this page" links.
           editUrl: 'https://github.com/raphamorim/rio/tree/main/docs/',
+          disableVersioning: false,
+          includeCurrentVersion: true,
+          lastVersion: undefined,
+          onlyIncludeVersions: ['current', '0.0.x'],
+          versions: {
+            current: {
+              label: '0.1.x',
+              path: 'next',
+              banner: 'none',
+            },
+            '0.0.x': {
+              label: '0.0.x',
+              path: '0.0.x',
+              banner: 'none',
+            },
+          },
         },
         blog: {
           showReadingTime: true,
@@ -80,6 +96,10 @@ const config = {
             require.resolve('./src/css/custom.css'),
           ],
         },
+        // gtag: {
+        //   trackingID: '---------',
+        //   anonymizeIP: true,
+        // },
       }),
     ],
   ],
@@ -94,15 +114,25 @@ const config = {
           src: '/assets/rio-logo.svg',
         },
         items: [
-          { to: '/docs/install', label: 'Install', position: 'left' },
-          { to: '/docs/configuration-file', label: 'Config', position: 'left' },
-          { to: '/docs/features', label: 'Features', position: 'left' },
+          { to: '/docs/next/install', label: 'Install', position: 'left' },
+          {
+            to: '/docs/next/configuration-file',
+            label: 'Config',
+            position: 'left',
+          },
+          { to: '/docs/next/features', label: 'Features', position: 'left' },
           { to: '/blog', label: 'Blog', position: 'left' },
           {
             href: 'https://discord.gg/zRvJjmKGwS',
             label: 'Discord',
             position: 'left',
           },
+          {
+            type: 'docsVersionDropdown',
+            position: 'right',
+            // dropdownItemsAfter: [{to: '/versions', label: 'All versions'}],
+            dropdownActiveClassDisabled: true,
+          },
           {
             type: 'localeDropdown',
             position: 'right',
@@ -123,15 +153,15 @@ const config = {
             items: [
               {
                 label: 'Install',
-                to: '/docs/install',
+                to: '/docs/next/install',
               },
               {
                 label: 'Config',
-                to: '/docs/configuration-file',
+                to: '/docs/next/configuration-file',
               },
               {
                 label: 'Features',
-                to: '/docs/features',
+                to: '/docs/next/features',
               },
             ],
           },
@@ -179,7 +209,7 @@ const config = {
       announcementBar: {
         id: 'support_us',
         content:
-          'If you use Rio terminal please consider support via <a target="_blank" rel="noopener noreferrer" href="https://github.com/sponsors/raphamorim">github sponsors</a>',
+          'Support Rio via <a target="_blank" rel="noopener noreferrer" href="https://github.com/sponsors/raphamorim">github sponsors</a>',
         backgroundColor: '#f712ff',
         textColor: '#FFFFFF',
         isCloseable: true,
