@@ -4,14 +4,14 @@ pub mod buffer;
 pub mod shapes;
 
 #[inline]
-pub fn orthographic_projection(width: u32, height: u32) -> [f32; 16] {
+pub fn orthographic_projection(width: f32, height: f32) -> [f32; 16] {
     [
-        2.0 / width as f32,
+        2.0 / width,
         0.0,
         0.0,
         0.0,
         0.0,
-        -2.0 / height as f32,
+        -2.0 / height,
         0.0,
         0.0,
         0.0,
