@@ -171,8 +171,8 @@ impl RichTextBrush {
             address_mode_u: wgpu::AddressMode::ClampToEdge,
             address_mode_v: wgpu::AddressMode::ClampToEdge,
             address_mode_w: wgpu::AddressMode::ClampToEdge,
-            mag_filter: wgpu::FilterMode::Nearest,
-            // mag_filter: wgpu::FilterMode::Linear,
+            // mag_filter: wgpu::FilterMode::Nearest,
+            mag_filter: wgpu::FilterMode::Linear,
             min_filter: wgpu::FilterMode::Nearest,
             mipmap_filter: wgpu::FilterMode::Nearest,
             lod_min_clamp: 0f32,
@@ -407,43 +407,11 @@ impl RichTextBrush {
             self.index_buffer_size = size;
         }
 
-        let mut ranges = vec![];
-
         let mut color_texture_updated: Option<&TextureId> = None;
         let mut mask_texture_updated: Option<&TextureId> = None;
 
         for command in self.dlist.commands() {
             match command {
-                Command::BindPipeline(pipeline) => {
-                    log::info!("BindPipeline {:?}", pipeline);
-                    // TODO:
-                    // rpass.set_blend_constant
-
-                    // match pipeline {
-                    //     Pipeline::Opaque => {
-                    //         unsafe {
-                    //             gl::DepthMask(1);
-                    //             gl::Disable(gl::BLEND);
-                    //             gl::BlendFunc(gl::SRC_ALPHA, gl::ONE_MINUS_SRC_ALPHA);
-                    //             gl::BlendEquation(gl::FUNC_ADD);
-                    //         }
-                    //     }
-                    //     Pipeline::Transparent => {
-                    //         unsafe {
-                    //             gl::DepthMask(0);
-                    //             gl::Enable(gl::BLEND);
-                    //             gl::BlendFunc(gl::SRC_ALPHA, gl::ONE_MINUS_SRC_ALPHA);
-                    //         }
-                    //     }
-                    //     Pipeline::Subpixel => {
-                    //         unsafe {
-                    //             gl::DepthMask(0);
-                    //             gl::Enable(gl::BLEND);
-                    //             gl::BlendFunc(gl::SRC1_COLOR, gl::ONE_MINUS_SRC1_COLOR);
-                    //         }
-                    //     }
-                    // }
-                }
                 Command::BindTexture(unit, id) => {
                     if self.bind_group_needs_update {
                         match unit {
@@ -477,10 +445,6 @@ impl RichTextBrush {
                         }
                     };
                 }
-                Command::Draw { start, count } => {
-                    let end = start + count;
-                    ranges.push((*start, end));
-                }
             }
         }
 
@@ -537,29 +501,16 @@ impl RichTextBrush {
             });
         }
 
-        // let mut rpass = encoder.begin_render_pass(&wgpu::RenderPassDescriptor {
-        //     label: None,
-        //     timestamp_writes: None,
-        //     occlusion_query_set: None,
-        //     color_attachments: &[Some(wgpu::RenderPassColorAttachment {
-        //         view,
-        //         resolve_target: None,
-        //         ops: wgpu::Operations {
-        //             load: wgpu::LoadOp::Load,
-        //             store: wgpu::StoreOp::Store,
-        //         },
-        //     })],
-        //     depth_stencil_attachment: None,
-        // });
         rpass.set_pipeline(&self.pipeline);
         rpass.set_bind_group(0, &self.bind_group, &[]);
         rpass.set_vertex_buffer(0, self.vertex_buffer.slice(..));
         rpass.set_index_buffer(self.index_buffer.slice(..), wgpu::IndexFormat::Uint32);
-        for items in ranges {
+
+        // Draw the specified range of indexed triangles.
+        for items in self.dlist.indices_to_draw() {
             rpass.draw_indexed(items.0..items.1, 0, 0..1);
         }
 
-        // drop(rpass);
         self.bind_group_needs_update = false;
         self.first_run = false;
     }
@@ -718,6 +669,7 @@ fn draw_layout(
                     let x = px + glyph.x;
                     let y = py - glyph.y;
                     px += glyph.advance;
+                    // px += rect.width;
                     glyphs.push(Glyph { id: glyph.id, x, y });
                 }
             }
