@@ -8,7 +8,7 @@
 
 //! Render data builder.
 
-use super::bidi::*;
+// use super::bidi::*;
 use super::builder_data::*;
 use super::span_style::*;
 use super::MAX_ID;
@@ -58,7 +58,7 @@ impl RunCache {
 pub struct LayoutContext {
     fcx: FontContext,
     fonts: FontLibrary,
-    bidi: BidiResolver,
+    // bidi: BidiResolver,
     scx: ShapeContext,
     state: BuilderState,
     cache: RunCache,
@@ -71,7 +71,7 @@ impl LayoutContext {
         Self {
             fonts: font_library.clone(),
             fcx: FontContext::default(),
-            bidi: BidiResolver::new(),
+            // bidi: BidiResolver::new(),
             scx: ShapeContext::new(),
             state: BuilderState::new(),
             cache: RunCache::new(),
@@ -89,7 +89,7 @@ impl LayoutContext {
     #[inline]
     pub fn builder(
         &mut self,
-        direction: Direction,
+        _direction: Direction,
         _language: Option<Language>,
         scale: f32,
     ) -> ParagraphBuilder {
@@ -98,9 +98,9 @@ impl LayoutContext {
         self.state.scale = scale;
         ParagraphBuilder {
             fcx: &mut self.fcx,
-            bidi: &mut self.bidi,
-            needs_bidi: false,
-            dir: direction,
+            // bidi: &mut self.bidi,
+            // needs_bidi: false,
+            // dir: direction,
             fonts: &self.fonts,
             scx: &mut self.scx,
             s: &mut self.state,
@@ -119,10 +119,10 @@ impl LayoutContext {
 /// Builder for computing the layout of a paragraph.
 pub struct ParagraphBuilder<'a> {
     fcx: &'a mut FontContext,
-    bidi: &'a mut BidiResolver,
+    // bidi: &'a mut BidiResolver,
     fonts: &'a FontLibrary,
-    needs_bidi: bool,
-    dir: Direction,
+    // needs_bidi: bool,
+    // dir: Direction,
     scx: &'a mut ShapeContext,
     s: &'a mut BuilderState,
     last_offset: u32,
@@ -187,6 +187,7 @@ impl<'a> ParagraphBuilder<'a> {
         }
 
         let mut offset = self.last_offset;
+        // let mut offset = self.last_offset;
         style.font_size *= self.s.scale;
         line.styles.push(style);
         let span_id = line.styles.len() - 1;
@@ -212,106 +213,111 @@ impl<'a> ParagraphBuilder<'a> {
         }
 
         let start = line.text.content.len();
-        match style.text_transform {
-            TextTransform::Uppercase => {
-                if let Some(lang) = &style.lang {
-                    match lang.language() {
-                        "tr" | "az" | "crh" | "tt" | "ba" => {
-                            for ch in text.chars() {
-                                match ch {
-                                    'i' => push_char!('İ'),
-                                    'ı' => push_char!('I'),
-                                    _ => {
-                                        for ch in ch.to_uppercase() {
-                                            push_char!(ch);
-                                        }
-                                    }
-                                }
-                            }
-                        }
-                        _ => {
-                            for ch in text.chars() {
-                                for ch in ch.to_uppercase() {
-                                    push_char!(ch);
-                                }
-                            }
-                        }
-                    }
-                } else {
-                    for ch in text.chars() {
-                        for ch in ch.to_uppercase() {
-                            push_char!(ch);
-                        }
-                    }
-                }
-            }
-            TextTransform::Lowercase => {
-                let mut iter = text.chars().peekable();
-                while let Some(ch) = iter.next() {
-                    if ch == 'Σ' {
-                        match iter.peek() {
-                            Some(ch) => {
-                                if ch.is_alphanumeric() || *ch == '-' {
-                                    push_char!('σ');
-                                } else {
-                                    push_char!('ς');
-                                }
-                            }
-                            None => {
-                                push_char!('ς');
-                            }
-                        }
-                    } else {
-                        for ch in ch.to_lowercase() {
-                            push_char!(ch);
-                        }
-                    }
-                }
-            }
-            TextTransform::Capitalize => {
-                let is_turkic = if let Some(lang) = &style.lang {
-                    matches!(lang.language(), "tr" | "az" | "crh" | "tt" | "ba")
-                } else {
-                    false
-                };
-                let mut cap_next = true;
-                for ch in text.chars() {
-                    if !ch.is_alphabetic() {
-                        // if ch.is_whitespace() || ch == '፡' {
-                        cap_next = true;
-                        push_char!(ch);
-                    } else if cap_next {
-                        if !ch.is_alphabetic() {
-                            push_char!(ch);
-                            continue;
-                        }
-                        if is_turkic {
-                            match ch {
-                                'i' => push_char!('İ'),
-                                'ı' => push_char!('I'),
-                                _ => {
-                                    for ch in ch.to_uppercase() {
-                                        push_char!(ch);
-                                    }
-                                }
-                            }
-                        } else {
-                            for ch in ch.to_uppercase() {
-                                push_char!(ch);
-                            }
-                        }
-                        cap_next = false;
-                    } else {
-                        push_char!(ch);
-                    }
-                }
-            }
-            _ => {
-                for ch in text.chars() {
-                    push_char!(ch);
-                }
-            }
+        // match style.text_transform {
+        //     TextTransform::Uppercase => {
+        //         if let Some(lang) = &style.lang {
+        //             match lang.language() {
+        //                 "tr" | "az" | "crh" | "tt" | "ba" => {
+        //                     for ch in text.chars() {
+        //                         match ch {
+        //                             'i' => push_char!('İ'),
+        //                             'ı' => push_char!('I'),
+        //                             _ => {
+        //                                 for ch in ch.to_uppercase() {
+        //                                     push_char!(ch);
+        //                                 }
+        //                             }
+        //                         }
+        //                     }
+        //                 }
+        //                 _ => {
+        //                     for ch in text.chars() {
+        //                         for ch in ch.to_uppercase() {
+        //                             push_char!(ch);
+        //                         }
+        //                     }
+        //                 }
+        //             }
+        //         } else {
+        //             for ch in text.chars() {
+        //                 for ch in ch.to_uppercase() {
+        //                     push_char!(ch);
+        //                 }
+        //             }
+        //         }
+        //     }
+        //     TextTransform::Lowercase => {
+        //         let mut iter = text.chars().peekable();
+        //         while let Some(ch) = iter.next() {
+        //             if ch == 'Σ' {
+        //                 match iter.peek() {
+        //                     Some(ch) => {
+        //                         if ch.is_alphanumeric() || *ch == '-' {
+        //                             push_char!('σ');
+        //                         } else {
+        //                             push_char!('ς');
+        //                         }
+        //                     }
+        //                     None => {
+        //                         push_char!('ς');
+        //                     }
+        //                 }
+        //             } else {
+        //                 for ch in ch.to_lowercase() {
+        //                     push_char!(ch);
+        //                 }
+        //             }
+        //         }
+        //     }
+        //     TextTransform::Capitalize => {
+        //         let is_turkic = if let Some(lang) = &style.lang {
+        //             matches!(lang.language(), "tr" | "az" | "crh" | "tt" | "ba")
+        //         } else {
+        //             false
+        //         };
+        //         let mut cap_next = true;
+        //         for ch in text.chars() {
+        //             if !ch.is_alphabetic() {
+        //                 // if ch.is_whitespace() || ch == '፡' {
+        //                 cap_next = true;
+        //                 push_char!(ch);
+        //             } else if cap_next {
+        //                 if !ch.is_alphabetic() {
+        //                     push_char!(ch);
+        //                     continue;
+        //                 }
+        //                 if is_turkic {
+        //                     match ch {
+        //                         'i' => push_char!('İ'),
+        //                         'ı' => push_char!('I'),
+        //                         _ => {
+        //                             for ch in ch.to_uppercase() {
+        //                                 push_char!(ch);
+        //                             }
+        //                         }
+        //                     }
+        //                 } else {
+        //                     for ch in ch.to_uppercase() {
+        //                         push_char!(ch);
+        //                     }
+        //                 }
+        //                 cap_next = false;
+        //             } else {
+        //                 push_char!(ch);
+        //             }
+        //         }
+        //     }
+        //     _ => {
+        //         for ch in text.chars() {
+        //             push_char!(ch);
+        //         }
+        //     }
+        // }
+
+        for ch in text.chars() {
+            push_char!(ch);
         }
+
         let end = line.text.content.len();
         let break_shaping = if let Some(prev_frag) = line.fragments.last() {
             let prev_style = line.styles[prev_frag.span];
@@ -320,8 +326,7 @@ impl<'a> ParagraphBuilder<'a> {
             } else {
                 style.font_size != prev_style.font_size
                     || style.letter_spacing != prev_style.letter_spacing
-                    || style.lang != prev_style.lang
-                    // || style.font != prev_stylefont
+                    // || style.lang != prev_style.lang
                     || style.font_features != prev_style.font_features
                     || style.font_vars != prev_style.font_vars
             }
@@ -412,25 +417,25 @@ impl<'a> ParagraphBuilder<'a> {
             for (props, boundary) in analysis.by_ref() {
                 line.text.info.push(CharInfo::new(props, boundary));
             }
-            if analysis.needs_bidi_resolution() || self.dir != Direction::LeftToRight {
-                let dir = match self.dir {
-                    Direction::Auto => None,
-                    Direction::LeftToRight => Some(BidiDirection::LeftToRight),
-                    Direction::RightToLeft => Some(BidiDirection::RightToLeft),
-                };
-                self.bidi.resolve_with_types(
-                    &self.s.lines[line_number].text.content,
-                    self.s.lines[line_number]
-                        .text
-                        .info
-                        .iter()
-                        .map(|i| i.bidi_class()),
-                    dir,
-                );
-                if !self.needs_bidi {
-                    self.needs_bidi = true;
-                }
-            }
+            // if analysis.needs_bidi_resolution() || self.dir != Direction::LeftToRight {
+            //     let dir = match self.dir {
+            //         Direction::Auto => None,
+            //         Direction::LeftToRight => Some(BidiDirection::LeftToRight),
+            //         Direction::RightToLeft => Some(BidiDirection::RightToLeft),
+            //     };
+            //     self.bidi.resolve_with_types(
+            //         &self.s.lines[line_number].text.content,
+            //         self.s.lines[line_number]
+            //             .text
+            //             .info
+            //             .iter()
+            //             .map(|i| i.bidi_class()),
+            //         dir,
+            //     );
+            //     if !self.needs_bidi {
+            //         self.needs_bidi = true;
+            //     }
+            // }
 
             self.itemize(line_number);
             self.shape(render_data, line_number);
@@ -450,7 +455,6 @@ impl<'a> ParagraphBuilder<'a> {
             self.cache.inner.clear();
             *render_data = RenderData::default();
             self.last_offset = 0;
-            self.needs_bidi = false;
 
             return self.resolve(render_data);
         };
@@ -471,13 +475,14 @@ impl<'a> ParagraphBuilder<'a> {
             .map(|i| i.script())
             .find(|s| real_script(*s))
             .unwrap_or(Script::Latin);
-        let levels = self.bidi.levels();
+        // let levels = self.bidi.levels();
         let mut last_frag = line.fragments.first().unwrap();
-        let mut last_level = if self.needs_bidi {
-            levels[last_frag.start]
-        } else {
-            0
-        };
+        // let mut last_level = if self.needs_bidi {
+        //     levels[last_frag.start]
+        // } else {
+        //     0
+        // };
+        let last_level = 0;
         let mut last_features = last_frag.features;
         let mut last_vars = last_frag.vars;
         let mut item = ItemData {
@@ -500,59 +505,59 @@ impl<'a> ParagraphBuilder<'a> {
                 }
             };
         }
-        if self.needs_bidi {
-            for frag in &line.fragments {
-                if frag.break_shaping || frag.start != last_frag.end {
+        // if self.needs_bidi {
+        //     for frag in &line.fragments {
+        //         if frag.break_shaping || frag.start != last_frag.end {
+        //             push_item!();
+        //             item.start = frag.start;
+        //             item.end = frag.start;
+        //         }
+        //         last_frag = frag;
+        //         last_features = frag.features;
+        //         last_vars = frag.vars;
+        //         let range = frag.start..frag.end;
+        //         for (&props, &level) in
+        //             line.text.info[range.clone()].iter().zip(&levels[range])
+        //         {
+        //             let script = props.script();
+        //             let real = real_script(script);
+        //             if (script != last_script && real) || level != last_level {
+        //                 //item.end += 1;
+        //                 push_item!();
+        //                 if real {
+        //                     last_script = script;
+        //                 }
+        //                 last_level = level;
+        //             }
+        //             item.end += 1;
+        //         }
+        //     }
+        // } else {
+        for frag in &line.fragments {
+            if frag.break_shaping || frag.start != last_frag.end {
+                push_item!();
+                item.start = frag.start;
+                item.end = frag.start;
+            }
+            last_frag = frag;
+            last_features = frag.features;
+            last_vars = frag.vars;
+            let range = frag.start..frag.end;
+            for &props in &line.text.info[range] {
+                let script = props.script();
+                let real = real_script(script);
+                if script != last_script && real {
+                    //item.end += 1;
                     push_item!();
-                    item.start = frag.start;
-                    item.end = frag.start;
-                }
-                last_frag = frag;
-                last_features = frag.features;
-                last_vars = frag.vars;
-                let range = frag.start..frag.end;
-                for (&props, &level) in
-                    line.text.info[range.clone()].iter().zip(&levels[range])
-                {
-                    let script = props.script();
-                    let real = real_script(script);
-                    if (script != last_script && real) || level != last_level {
-                        //item.end += 1;
-                        push_item!();
-                        if real {
-                            last_script = script;
-                        }
-                        last_level = level;
+                    if real {
+                        last_script = script;
                     }
+                } else {
                     item.end += 1;
                 }
             }
-        } else {
-            for frag in &line.fragments {
-                if frag.break_shaping || frag.start != last_frag.end {
-                    push_item!();
-                    item.start = frag.start;
-                    item.end = frag.start;
-                }
-                last_frag = frag;
-                last_features = frag.features;
-                last_vars = frag.vars;
-                let range = frag.start..frag.end;
-                for &props in &line.text.info[range] {
-                    let script = props.script();
-                    let real = real_script(script);
-                    if script != last_script && real {
-                        //item.end += 1;
-                        push_item!();
-                        if real {
-                            last_script = script;
-                        }
-                    } else {
-                        item.end += 1;
-                    }
-                }
-            }
         }
+        // }
         push_item!();
     }
 
@@ -646,10 +651,10 @@ fn shape_item(
     };
 
     if item.level & 1 != 0 {
-        let chars = state.lines[current_line].text.content[range.clone()]
+        let chars = state.lines[current_line].text.content[range.to_owned()]
             .iter()
-            .zip(&state.lines[current_line].text.offsets[range.clone()])
-            .zip(&state.lines[current_line].text.spans[range.clone()])
+            .zip(&state.lines[current_line].text.offsets[range.to_owned()])
+            .zip(&state.lines[current_line].text.spans[range.to_owned()])
             .zip(&state.lines[current_line].text.info[range])
             .map(|z| {
                 use swash::text::Codepoint;
@@ -689,10 +694,10 @@ fn shape_item(
             cache.insert(line_hash, render_data.last_cached_run.to_owned());
         }
     } else {
-        let chars = state.lines[current_line].text.content[range.clone()]
+        let chars = state.lines[current_line].text.content[range.to_owned()]
             .iter()
-            .zip(&state.lines[current_line].text.offsets[range.clone()])
-            .zip(&state.lines[current_line].text.spans[range.clone()])
+            .zip(&state.lines[current_line].text.offsets[range.to_owned()])
+            .zip(&state.lines[current_line].text.spans[range.to_owned()])
             .zip(&state.lines[current_line].text.info[range])
             .map(|z| {
                 let (((&ch, &offset), &span_index), &info) = z;
@@ -760,7 +765,7 @@ where
     let mut shaper = scx
         .builder(fonts[current_font_id].as_ref())
         .script(state.script)
-        .language(state.span.lang)
+        // .language(state.span.lang)
         .direction(dir)
         .size(state.size)
         .features(state.features.iter().copied())
@@ -770,7 +775,14 @@ where
 
     let mut synth = Synthesis::default();
     loop {
+        // for c in cluster.chars().iter() {
+        //     let width = c.ch.width().unwrap_or(1);
+        //     if width > 1 {
+        //         println!("{:?} {}", c, c.ch.width().unwrap_or(1));
+        //     }
+        // }
         shaper.add_cluster(cluster);
+
         if !parser.next(cluster) {
             render_data.push_run(
                 &state.state.lines[current_line].styles,
